import { Injectable, signal, WritableSignal } from '@angular/core';
import { Repo, Settings } from '../interface/repo'
import { Response } from "../interface/response";
import { HttpClient } from "@angular/common/http";
import { MsgService } from "./msg.service";

@Injectable({
  providedIn: 'root'
})
export class RepoService {

  readonly init_catalog = [
    { id: 1, created_at: 1713164091576, updated_at: 1713164091576, digest_id: 1, repo: "external/golang", tag: "latest" },
    { id: 2, created_at: 1713164091576, updated_at: 1713164091576, digest_id: 2, repo: "external/alpine", tag: "latest" },
    { id: 3, created_at: 1713164091576, updated_at: 1713164091576, digest_id: 3, repo: "external/nginx", tag: "latest" },
  ] as Repo[]

  readonly url_settings = '/_api/repo/settings'
  readonly url_catalog = '/_api/repo/list'
  readonly url_tag = '/_api/repo/tag/list'

  settings = signal({} as Settings)

  catalog = signal({
    total: 0,
    limit: 0,
    last: 0,
    keyword: '',
    list: [] as Repo[],
  })

  constructor(
    private http: HttpClient,
    private msg_svc: MsgService,
  ) {
    this.GetSettings()
    this.GetCatalog()
  }

  GetSettings() {
    this.http.get<Response<Settings>>(this.url_settings).subscribe(res => {
      if (res.status === 200) {
        this.settings.update(v => res.data)
        return
      }

      this.msg_svc.error(res.msg)
    })
  }

  GetCatalog(limit: number = 20, last: number = 0, keyword: string = '') {
    this.http.get<Response<{ total: number, list: Repo[] }>>(this.url_catalog, {
      params: {
        n: limit,
        last: last,
        keyword: keyword
      }
    }).subscribe(res => {
      if (res.status === 200) {
        this.catalog.update(v => {
          return { ...v, limit: limit, last: last, keyword: keyword, list: res.data.list, total: res.data.total }
        })

        return
      }

      this.msg_svc.error(res.msg)
    })
  }

  GetTags(repo: string, limit: number = 20, last: number = 0, keyword: string = '') {
    console.log(`[D] GetTags: repo=${repo} limit=${limit} last=${last} keyword=${keyword}`)

    this.http.get<Response<{ total: number, list: Repo[] }>>(this.url_tag, {
      params: {
        repo: repo,
        n: limit,
        last: last,
        keyword: keyword
      }
    }).subscribe(res => {
      if (res.status === 200) {
        this.catalog().list.forEach(v => {
          if (v.repo === repo) {
            v.Tags = {
              list: res.data.list,
              total: res.data.total,
              limit: limit,
              last: last,
              keyword: keyword,
            }
          }
        })

        return
      }

      this.msg_svc.error(res.msg)
    })
  }
}
