package verify

import (
	"bytes"
	"crypto/sha256"
	"encoding/hex"
	"fmt"
	"io"
	"nf-repo/internal/model"
	"os"
	"testing"
)

func TestVerify(t *testing.T) {
	bs := []byte(`{"architecture":"amd64","config":{"Hostname":"","Domainname":"","User":"","AttachStdin":false,"AttachStdout":false,"AttachStderr":false,"Tty":false,"OpenStdin":false,"StdinOnce":false,"Env":["PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"],"Cmd":["/bin/sh"],"Image":"sha256:9a5ce069f40cfe0f2270eafbff0a0f2fa08f1add73571af9f78209e96bb8a5e9","Volumes":null,"WorkingDir":"","Entrypoint":null,"OnBuild":null,"Labels":null},"container":"4189cbc534955765760c227f328ec1cdd52e8550681c2bf9f8f990b27b644f9c","container_config":{"Hostname":"4189cbc53495","Domainname":"","User":"","AttachStdin":false,"AttachStdout":false,"AttachStderr":false,"Tty":false,"OpenStdin":false,"StdinOnce":false,"Env":["PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"],"Cmd":["/bin/sh","-c","#(nop) ","CMD [\"/bin/sh\"]"],"Image":"sha256:9a5ce069f40cfe0f2270eafbff0a0f2fa08f1add73571af9f78209e96bb8a5e9","Volumes":null,"WorkingDir":"","Entrypoint":null,"OnBuild":null,"Labels":{}},"created":"2024-01-27T00:30:48.743965523Z","docker_version":"20.10.23","history":[{"created":"2024-01-27T00:30:48.624602109Z","created_by":"/bin/sh -c #(nop) ADD file:37a76ec18f9887751cd8473744917d08b7431fc4085097bb6a09d81b41775473 in / "},{"created":"2024-01-27T00:30:48.743965523Z","created_by":"/bin/sh -c #(nop)  CMD [\"/bin/sh\"]","empty_layer":true}],"os":"linux","rootfs":{"type":"layers","diff_ids":["sha256:d4fc045c9e3a848011de66f34b81f052d4f2c15a17bb196d637e526349601820"]}}`)
	//hash, _ := model.NewHash("sha256:05455a08881ea9cf0e752bc48e61bbd71a34c029bb13df01e40e3e70e0d007bd")
	badHash, _ := model.NewHash("sha256:12455a08881ea9cf0e7f2bc48e61bbd71a34c029bb13df01e40e3e70e0d007bd")
	nrc, err := ReadCloser(io.NopCloser(bytes.NewReader(bs)), -1, badHash)
	if err != nil {
		t.Error(1, err)
		return
	}

	nbs, err := io.ReadAll(nrc)
	if err != nil {
		t.Error(2, err)
	}

	if string(nbs) != string(bs) {
		t.Error(3, "not same")
	}

	t.Log("new bytes:", string(nbs))
}

func TestVerify2(t *testing.T) {
	name := "4a666f159bd07f62aa19ce8eceac018931223de67c4bd19ded87b83eb7b103ca"
	f, err := os.Open(name)
	if err != nil {
		t.Error(1, err)
		return
	}

	bs, err := io.ReadAll(f)
	if err != nil {
		t.Error(2, err)
	}

	shaer := sha256.New()
	shaer.Write(bs)
	sum := hex.EncodeToString(shaer.Sum(nil))

	r, err := ReadCloser(io.NopCloser(bytes.NewReader(bs)), int64(len(bs)), model.Hash{Algorithm: "sha256", Hex: name})
	if err != nil {
		t.Error(4, err)
		return
	}

	if _, err = io.ReadAll(r); err != nil {
		t.Error(5, err)
		return
	}

	if sum != name {
		t.Error(3, fmt.Sprintf("want: %s got: %s", name, sum))
	}

	_ = r
}
